@@ -3,7 +3,8 @@ import time as _time
 from threading import Event as _Flag
 import numpy as _np
 
-from .device import Device as _Device, Devices as _Devices
+from .device import Device as _Device, Devices as _Devices, \
+    ProptyDevice as _ProptyDevice
 from ..diagbeam.bpm.csdev import Const as _csbpm
 from ..search import BPMSearch as _BPMSearch
 from ..namesys import SiriusPVName as _PVName
@@ -823,8 +824,8 @@ class FamBPMs(_Devices):
         devs = [BPM(dev, auto_mon=False) for dev in bpm_names]
 
         super().__init__(devname, devs)
-        self.bpm_names = bpm_names
-        self.csbpm = devs[0].csdata
+        self._bpm_names = bpm_names
+        self._csbpm = devs[0].csdata
         propties_to_keep = ['GEN_XArrayData', 'GEN_YArrayData']
 
         self._mturn_flags = dict()
@@ -835,6 +836,16 @@ class FamBPMs(_Devices):
                 self._mturn_flags[pvo.pvname] = _Flag()
                 pvo.add_callback(self._mturn_set_flag)
 
+    @property
+    def bpm_names(self):
+        """Return BPM names."""
+        return self._bpm_names
+
+    @property
+    def csbpm(self):
+        """Return control system BPM constants class."""
+        return self._csbpm
+
     def set_attenuation(self, value=RFFEATT_MAX, timeout=TIMEOUT):
         """."""
         for bpm in self:
@@ -936,23 +947,23 @@ class FamBPMs(_Devices):
 
         """
         if acq_rate.lower().startswith('monit1'):
-            acq_rate = self.csbpm.AcqChan.Monit1
+            acq_rate = self._csbpm.AcqChan.Monit1
         elif acq_rate.lower().startswith('fofb'):
-            acq_rate = self.csbpm.AcqChan.FOFB
+            acq_rate = self._csbpm.AcqChan.FOFB
         elif acq_rate.lower().startswith('tbt'):
-            acq_rate = self.csbpm.AcqChan.TbT
+            acq_rate = self._csbpm.AcqChan.TbT
         else:
             raise ValueError(acq_rate + ' is not a valid acquisition rate.')
 
         if repeat:
-            repeat = self.csbpm.AcqRepeat.Repetitive
+            repeat = self._csbpm.AcqRepeat.Repetitive
         else:
-            repeat = self.csbpm.AcqRepeat.Normal
+            repeat = self._csbpm.AcqRepeat.Normal
 
         if external:
-            trig = self.csbpm.AcqTrigTyp.External
+            trig = self._csbpm.AcqTrigTyp.External
         else:
-            trig = self.csbpm.AcqTrigTyp.Now
+            trig = self._csbpm.AcqTrigTyp.Now
 
         self.cmd_mturn_acq_abort()
 
@@ -973,7 +984,7 @@ class FamBPMs(_Devices):
 
         """
         for bpm in self._devices:
-            bpm.acq_ctrl = self.csbpm.AcqEvents.Abort
+            bpm.acq_ctrl = self._csbpm.AcqEvents.Abort
 
         for bpm in self._devices:
             boo = bpm.wait_acq_finish()
@@ -989,7 +1000,7 @@ class FamBPMs(_Devices):
 
         """
         for bpm in self._devices:
-            bpm.acq_ctrl = self.csbpm.AcqEvents.Start
+            bpm.acq_ctrl = self._csbpm.AcqEvents.Start
 
         for bpm in self._devices:
             boo = bpm.wait_acq_start()
@@ -997,6 +1008,23 @@ class FamBPMs(_Devices):
                 return False
         return True
 
+    def set_switching_mode(self, mode='direct'):
+        """Set switching mode of BPMS.
+
+        Args:
+            mode ((str, int), optional): Desired mode, must be in
+                {'direct', 'switching', 1, 3}. Defaults to 'direct'.
+
+        Raises:
+            ValueError: When mode is not in {'direct', 'switching', 1, 3}.
+
+        """
+        if mode not in ('direct', 'switching', 1, 3):
+            raise ValueError('Value must be in ("direct", "switching", 1, 3).')
+
+        for bpm in self._devices:
+            bpm.switching_mode = mode
+
     def mturn_reset_flags(self):
         """Reset Multiturn flags to wait for a new orbit update."""
         for flag in self._mturn_flags.values():
@@ -1059,3 +1087,60 @@ class FamBPMs(_Devices):
     def _mturn_set_flag(self, pvname, **kwargs):
         _ = kwargs
         self._mturn_flags[pvname].set()
+
+
+class BPMLogicalTrigger(_ProptyDevice):
+    """BPM Logical Trigger device."""
+
+    _properties = (
+        'RcvSrc-Sel', 'RcvSrc-Sts',
+        'RcvInSel-SP', 'RcvInSel-RB',
+        'TrnSrc-Sel', 'TrnSrc-Sts',
+        'TrnOutSel-SP', 'TrnOutSel-RB',
+    )
+
+    def __init__(self, bpmname, index):
+        """Init."""
+        if not _BPMSearch.is_valid_devname(bpmname):
+            raise NotImplementedError(bpmname)
+        if not 0 <= int(index) <= 23:
+            raise NotImplementedError(index)
+        super().__init__(
+            bpmname, 'TRIGGER'+str(index),
+            properties=BPMLogicalTrigger._properties)
+
+    @property
+    def receiver_source(self):
+        """Receiver source."""
+        return self['RcvSrc-Sts']
+
+    @receiver_source.setter
+    def receiver_source(self, value):
+        self['RcvSrc-Sel'] = value
+
+    @property
+    def receiver_in_sel(self):
+        """Receiver in selection."""
+        return self['RcvInSel-RB']
+
+    @receiver_in_sel.setter
+    def receiver_in_sel(self, value):
+        self['RcvInSel-SP'] = value
+
+    @property
+    def transmitter_source(self):
+        """Transmitter source."""
+        return self['TrnSrc-Sts']
+
+    @transmitter_source.setter
+    def transmitter_source(self, value):
+        self['TrnSrc-Sel'] = value
+
+    @property
+    def transmitter_out_sel(self):
+        """Transmitter out selection."""
+        return self['TrnOutSel-RB']
+
+    @transmitter_out_sel.setter
+    def transmitter_out_sel(self, value):
+        self['TrnOutSel-SP'] = value
