@@ -7,7 +7,7 @@ from mathphys.functions import get_namedtuple as _get_namedtuple
 
 from ..namesys import SiriusPVName as _PVName
 from ..search import BPMSearch as _BPMSearch, PSSearch as _PSSearch
-from ..fofb.csdev import HLFOFBConst as _Const
+from ..fofb.csdev import HLFOFBConst as _Const, NR_BPM
 
 from .device import Device as _Device, ProptyDevice as _ProptyDevice, \
     Devices as _Devices
@@ -16,8 +16,6 @@ from .timing import Event
 from .pwrsupply import PowerSupplyFC
 from .psconv import StrengthConv
 
-NR_BPM = 160
-
 
 class _FOFBCtrlBase:
     """FOFB Ctrl base."""
@@ -28,10 +26,10 @@ class _FOFBCtrlBase:
 
 
 class FOFBCtrlRef(_Device, _FOFBCtrlBase):
-    """FOFB controller device."""
+    """FOFB reference orbit controller device."""
 
     _properties = (
-        'RefOrbit-SP', 'RefOrbit-RB',
+        'RefOrb-SP', 'RefOrb-RB',
     )
 
     def __init__(self, devname):
@@ -46,17 +44,20 @@ class FOFBCtrlRef(_Device, _FOFBCtrlBase):
     @property
     def ref(self):
         """Reference orbit, first half reference for X, second, for Y."""
-        ref = self['RefOrbit-RB']
-        # handle initial state of RefOrbit PVs
+        ref = self['RefOrb-RB']
+        if ref is None:
+            return None
+        ref = ref.copy()
+        # handle initial state of RefOrb PVs
         if len(ref) < 2*NR_BPM:
-            value = _np.zeros(2*NR_BPM)
+            value = _np.zeros(2*NR_BPM, dtype=int)
             value[:len(ref)] = ref
             ref = value
         return ref
 
     @ref.setter
     def ref(self, value):
-        self['RefOrbit-SP'] = _np.array(value, dtype=int)
+        self['RefOrb-SP'] = _np.array(value, dtype=int)
 
     @property
     def refx(self):
@@ -80,12 +81,33 @@ class FOFBCtrlRef(_Device, _FOFBCtrlBase):
         var[NR_BPM:] = _np.array(value, dtype=int)
         self.ref = var
 
+    def set_ref(self, value):
+        """Set RefOrb."""
+        self.ref = value
+        return True
+
+    def set_refx(self, value):
+        """Set RefOrb X."""
+        self.refx = value
+        return True
+
+    def set_refy(self, value):
+        """Set RefOrb Y."""
+        self.refy = value
+        return True
+
+    def check_ref(self, value):
+        """Check whether RefOrb is equal to value."""
+        if not _np.all(self.ref == value):
+            return False
+        return True
+
     def check_refx(self, value):
-        """Check if first half of RefOrb is equal to value."""
+        """Check whether first half of RefOrb is equal to value."""
         return self._check_reforbit('x', value)
 
     def check_refy(self, value):
-        """Check if second half of RefOrb is equal to value."""
+        """Check whether second half of RefOrb is equal to value."""
         return self._check_reforbit('y', value)
 
     def _check_reforbit(self, plane, value):
@@ -96,29 +118,48 @@ class FOFBCtrlRef(_Device, _FOFBCtrlBase):
 
 
 class _DCCDevice(_ProptyDevice):
+    """FOFB Diamond communication controller device."""
 
     DEF_TIMEOUT = 1
-    DEF_FMC_BPMCNT = 160
+    DEF_FMC_BPMCNT = NR_BPM
     DEF_P2P_BPMCNT = 8
 
+    _sync_sleep = 1.0  # [s]
+
     _properties = (
-        'BPMId-RB', 'BPMCnt-Mon',
+        'BPMId-SP', 'BPMId-RB', 'BPMCnt-Mon',
         'CCEnable-SP', 'CCEnable-RB',
         'TimeFrameLen-SP', 'TimeFrameLen-RB',
     )
+    _properties_fmc = (
+        'LinkPartnerCH0-Mon', 'LinkPartnerCH1-Mon',
+        'LinkPartnerCH2-Mon', 'LinkPartnerCH3-Mon',
+    )
 
     def __init__(self, devname, dccname):
         """Init."""
         self.dccname = dccname
-        super().__init__(
-            devname, dccname, properties=_DCCDevice._properties)
-        self.set_auto_monitor('BPMCnt-Mon', True)
+
+        properties = _DCCDevice._properties
+        if 'FMC' in self.dccname:
+            properties += _DCCDevice._properties_fmc
+
+        super().__init__(devname, dccname, properties=properties)
+        prop2automon = [
+            'BPMCnt-Mon', 'LinkPartnerCH0-Mon', 'LinkPartnerCH1-Mon',
+            'LinkPartnerCH2-Mon', 'LinkPartnerCH3-Mon']
+        for prop in prop2automon:
+            self.set_auto_monitor(prop, True)
 
     @property
     def bpm_id(self):
         """BPM Id."""
         return self['BPMId-RB']
 
+    @bpm_id.setter
+    def bpm_id(self, value):
+        self['BPMId-SP'] = value
+
     @property
     def bpm_count(self):
         """BPM count."""
@@ -151,15 +192,6 @@ class _DCCDevice(_ProptyDevice):
             else self.DEF_P2P_BPMCNT
         return self['BPMCnt-Mon'] == cnt
 
-    def cmd_sync(self, timeout=DEF_TIMEOUT):
-        """Synchronize DCC."""
-        self.cc_enable = 0
-        if not self._wait('CCEnable-RB', 0, timeout/2):
-            return False
-        _time.sleep(1)
-        self.cc_enable = 1
-        return self._wait('CCEnable-RB', 1, timeout/2)
-
 
 class FOFBCtrlDCC(_DCCDevice, _FOFBCtrlBase):
     """FOFBCtrl DCC device."""
@@ -179,6 +211,14 @@ class FOFBCtrlDCC(_DCCDevice, _FOFBCtrlBase):
             raise NotImplementedError(dccname)
         super().__init__(devname, dccname)
 
+    @property
+    def linkpartners(self):
+        """Return linked partners."""
+        linkpart_props = [
+            'LinkPartnerCH0-Mon', 'LinkPartnerCH1-Mon',
+            'LinkPartnerCH2-Mon', 'LinkPartnerCH3-Mon']
+        return set(self[prop] for prop in linkpart_props)
+
 
 class BPMDCC(_DCCDevice):
     """BPM DCC device."""
@@ -203,18 +243,23 @@ class FamFOFBControllers(_Devices):
     def __init__(self):
         """Init."""
         # FOFBCtrl Refs and DCCs
-        self._ctl_refs, self._ctl_dccs, self._ctl_ids = dict(), dict(), dict()
-        for ctl in _FOFBCtrlBase.DEVICES:
-            self._ctl_ids[ctl] = self.FOFBCTRL_BPMID_OFFSET-1+int(ctl[3:5])
+        bpmids = _np.array(
+            [self.FOFBCTRL_BPMID_OFFSET - 1 + i for i in range(1, 21)])
+        lpcw = _np.roll(bpmids, 1)
+        lpaw = _np.roll(bpmids, -1)
+        self._ctl_ids, self._ctl_part = dict(), dict()
+        self._ctl_refs, self._ctl_dccs = dict(), dict()
+        for idx, ctl in enumerate(_FOFBCtrlBase.DEVICES):
+            self._ctl_ids[ctl] = bpmids[idx]
+            self._ctl_part[ctl] = {lpcw[idx], lpaw[idx]}
             self._ctl_refs[ctl] = FOFBCtrlRef(ctl)
             for dcc in FOFBCtrlDCC.PROPDEVICES.ALL:
-                self._ctl_dccs[ctl+':'+dcc] = FOFBCtrlDCC(ctl, dcc)
+                self._ctl_dccs[ctl + ':' + dcc] = FOFBCtrlDCC(ctl, dcc)
         # BPM DCCs and triggers
-        bpmnames = _BPMSearch.get_names({'sec': 'SI', 'dev': 'BPM'})
-        bpmids = _np.roll(
-            _np.array([i-1 if i % 2 == 1 else i for i in range(160)]), -1)
+        self._bpmnames = _BPMSearch.get_names({'sec': 'SI', 'dev': 'BPM'})
+        bpmids = [((i + 1) // 2) * 2 % 160 for i in range(NR_BPM)]
         self._bpm_dccs, self._bpm_trgs, self._bpm_ids = dict(), dict(), dict()
-        for idx, bpm in enumerate(bpmnames):
+        for idx, bpm in enumerate(self._bpmnames):
             self._bpm_ids[bpm] = bpmids[idx]
             self._bpm_dccs[bpm] = BPMDCC(bpm)
             for trig in self.BPM_TRIGS_IDS:
@@ -232,6 +277,12 @@ class FamFOFBControllers(_Devices):
 
         super().__init__('SI-Glob:BS-FOFB', devices)
 
+    def set_reforb(self, value):
+        """Set RefOrb for all FOFB controllers."""
+        for ctrl in self._ctl_refs.values():
+            ctrl.set_ref(value)
+        return True
+
     def set_reforbx(self, value):
         """Set RefOrbX for all FOFB controllers."""
         return self._set_reforb('x', value)
@@ -242,7 +293,17 @@ class FamFOFBControllers(_Devices):
 
     def _set_reforb(self, plane, value):
         for ctrl in self._ctl_refs.values():
-            setattr(ctrl, 'ref'+plane.lower(), value)
+            fun = getattr(ctrl, 'set_ref' + plane.lower())
+            fun(value)
+        return True
+
+    def check_reforb(self, value):
+        """Check whether RefOrb is equal to value."""
+        if not self.connected:
+            return False
+        for ctrl in self._ctl_refs.values():
+            if not ctrl.check_ref(value):
+                return False
         return True
 
     def check_reforbx(self, value):
@@ -254,8 +315,10 @@ class FamFOFBControllers(_Devices):
         return self._check_reforb('y', value)
 
     def _check_reforb(self, plane, value):
+        if not self.connected:
+            return False
         for ctrl in self._ctl_refs.values():
-            fun = getattr(ctrl, 'check_ref'+plane.lower())
+            fun = getattr(ctrl, 'check_ref' + plane.lower())
             if not fun(value):
                 return False
         return True
@@ -277,13 +340,51 @@ class FamFOFBControllers(_Devices):
         """Check whether DCC BPMIds are configured."""
         if not self.connected:
             return False
-        isconf = True
         for dcc, dev in self._ctl_dccs.items():
             ctl = _PVName(dcc).device_name
-            isconf &= dev.bpm_id == self._ctl_ids[ctl]
+            if not dev.bpm_id == self._ctl_ids[ctl]:
+                return False
         for bpm, dev in self._bpm_dccs.items():
-            isconf &= dev.bpm_id == self._bpm_ids[bpm]
-        return isconf
+            if not dev.bpm_id == self._bpm_ids[bpm]:
+                return False
+        return True
+
+    def cmd_config_bpm_id(self):
+        """Command to configure DCC BPMIds."""
+        if not self.connected:
+            return False
+        for dcc, dev in self._ctl_dccs.items():
+            ctl = _PVName(dcc).device_name
+            dev.bpm_id = self._ctl_ids[ctl]
+        for bpm, dev in self._bpm_dccs.items():
+            dev.bpm_id = self._bpm_ids[bpm]
+        return True
+
+    @property
+    def linkpartners(self):
+        """Return link partners."""
+        if not self.connected:
+            return False
+        partners = dict()
+        for dev in self._ctl_dccs.values():
+            if 'FMC' not in dev.dccname:
+                continue
+            partners[dev.devname] = dev.linkpartners
+        return partners
+
+    @property
+    def linkpartners_connected(self):
+        """Check whether adjacent partners are connected."""
+        if not self.connected:
+            return False
+        for dcc, dev in self._ctl_dccs.items():
+            if 'FMC' not in dev.dccname:
+                continue
+            ctl = _PVName(dcc).device_name
+            nrpart = len(self._ctl_part[ctl])
+            if not len(dev.linkpartners & self._ctl_part[ctl]) == nrpart:
+                return False
+        return True
 
     @property
     def bpm_count(self):
@@ -297,30 +398,40 @@ class FamFOFBControllers(_Devices):
             bpmids[dev.pv_object('BPMCnt-Mon').pvname] = dev.bpm_count
         return bpmids
 
-    @property
-    def net_synced(self):
+    def cmd_sync_net(self, bpms=None, timeout=DEF_TIMEOUT):
+        """Command to synchronize DCCs."""
+        alldccs = list(self._ctl_dccs.values()) + list(self._bpm_dccs.values())
+        enbdccs = list(self._ctl_dccs.values())
+        if bpms is None:
+            bpms = self._bpmnames
+        for bpm in bpms:
+            enbdccs.append(self._bpm_dccs[bpm])
+
+        self._set_devices_propty(alldccs, 'CCEnable-SP', 0)
+        if not self._wait_devices_propty(
+                alldccs, 'CCEnable-RB', 0, timeout=timeout/2):
+            return False
+        self._set_devices_propty(enbdccs, 'CCEnable-SP', 1)
+        if not self._wait_devices_propty(
+                enbdccs, 'CCEnable-RB', 1, timeout=timeout/2):
+            return False
+        self._evt_fofb.cmd_external_trigger()
+        return True
+
+    def check_net_synced(self, bpms=None):
         """Check whether DCCs are synchronized."""
         if not self.connected:
             return False
-        issync = True
+        if bpms is None:
+            bpms = self._bpmnames
         for dev in self._ctl_dccs.values():
-            issync &= dev.is_synced
-        for dev in self._bpm_dccs.values():
-            issync &= dev.is_synced
-        return issync
-
-    def cmd_sync_net(self, timeout=DEF_TIMEOUT):
-        """Command to synchronize DCCs."""
-        devs = list(self._ctl_dccs.values()) + list(self._bpm_dccs.values())
-        self._set_devices_propty(devs, 'CCEnable-SP', 0)
-        if not self._wait_devices_propty(
-                devs, 'CCEnable-RB', 0, timeout=timeout/2):
-            return False
-        self._set_devices_propty(devs, 'CCEnable-SP', 1)
-        if not self._wait_devices_propty(
-                devs, 'CCEnable-RB', 1, timeout=timeout/2):
-            return False
-        self._evt_fofb.cmd_external_trigger()
+            if dev.dccname != 'DCCFMC':
+                continue
+            if not dev.bpm_count == len(bpms):
+                return False
+        for bpm in bpms:
+            if not self._bpm_dccs[bpm].cc_enable == 1:
+                return False
         return True
 
     @property
@@ -361,11 +472,12 @@ class FamFOFBControllers(_Devices):
         """Check whether all BPM triggers are configured."""
         if not self.connected:
             return False
-        isconf = True
         for dev in self._bpm_trgs.values():
-            isconf &= dev.receiver_source == self.DEF_BPMTRIG_RCVSRC
-            isconf &= dev.receiver_in_sel == self.DEF_BPMTRIG_RCVIN
-        return isconf
+            if not dev.receiver_source == self.DEF_BPMTRIG_RCVSRC:
+                return False
+            if not dev.receiver_in_sel == self.DEF_BPMTRIG_RCVIN:
+                return False
+        return True
 
     def cmd_config_bpm_trigs(self, timeout=DEF_TIMEOUT):
         """Command to configure BPM triggers."""
@@ -391,6 +503,9 @@ class FamFastCorrs(_Devices):
     OPMODE_STS = PowerSupplyFC.OPMODE_STS
     DEF_ATOL_INVRESPMATROW = 2**-17
     DEF_ATOL_FOFBACCGAIN = 2**-12
+    DEF_ATOL_FOFBACCSAT = 2e-2
+    DEF_ATOL_CURRENT_RB = 1e-6
+    DEF_ATOL_CURRENT_MON = 2e-2
 
     def __init__(self, psnames=None):
         """Init."""
@@ -409,6 +524,16 @@ class FamFastCorrs(_Devices):
         """PS name list."""
         return list(self._psnames)
 
+    @property
+    def psdevs(self):
+        """PS device list."""
+        return self._psdevs
+
+    @property
+    def psconvs(self):
+        """PS conversion device list."""
+        return self._psconv
+
     @property
     def pwrstate(self):
         """Power State.
@@ -429,6 +554,26 @@ class FamFastCorrs(_Devices):
         """
         return _np.array([p.opmode for p in self._psdevs])
 
+    @property
+    def current(self):
+        """Current readback.
+
+        Returns:
+            current (numpy.ndarray, 160):
+                OpMode for each power supply.
+        """
+        return _np.array([p.current for p in self._psdevs])
+
+    @property
+    def current_mon(self):
+        """Implemented current.
+
+        Returns:
+            current (numpy.ndarray, 160):
+                OpMode for each power supply.
+        """
+        return _np.array([p.current_mon for p in self._psdevs])
+
     @property
     def fofbacc_gain(self):
         """FOFB pre-accumulator gain.
@@ -449,6 +594,28 @@ class FamFastCorrs(_Devices):
         """
         return _np.array([p.fofbacc_freeze for p in self._psdevs])
 
+    @property
+    def fofbacc_satmax(self):
+        """FOFB pre-accumulator maximum saturation current [A].
+
+        Returns:
+            gain (numpy.ndarray, 160):
+                FOFB pre-accumulator maximum saturation current
+                for each power supply.
+        """
+        return _np.array([p.fofbacc_satmax for p in self._psdevs])
+
+    @property
+    def fofbacc_satmin(self):
+        """FOFB pre-accumulator minimum saturation current [A].
+
+        Returns:
+            gain (numpy.ndarray, 160):
+                FOFB pre-accumulator minimum saturation current
+                for each power supply.
+        """
+        return _np.array([p.fofbacc_satmin for p in self._psdevs])
+
     @property
     def curr_gain(self):
         """Current gain.
@@ -464,7 +631,7 @@ class FamFastCorrs(_Devices):
         """Strength to current convertion factor.
 
         Returns:
-            factor (numpy.ndarray, 160):
+            factor (numpy.ndarray, NR_BPM):
                 convertion factor for each power supply.
         """
         return _np.array(
@@ -482,6 +649,8 @@ class FamFastCorrs(_Devices):
             self, state, psnames=None, psindices=None,
             timeout=DEF_TIMEOUT):
         """Check whether power supplies are in desired pwrstate."""
+        if not self.connected:
+            return False
         devs = self._get_devices(psnames, psindices)
         return self._wait_devices_propty(
             devs, 'PwrState-Sts', state, timeout=timeout)
@@ -497,10 +666,49 @@ class FamFastCorrs(_Devices):
             self, opmode, psnames=None, psindices=None,
             timeout=DEF_TIMEOUT):
         """Check whether power supplies are in desired opmode."""
+        if not self.connected:
+            return False
         devs = self._get_devices(psnames, psindices)
         return self._wait_devices_propty(
             devs, 'OpMode-Sts', opmode, timeout=timeout)
 
+    def set_current(self, values, psnames=None, psindices=None):
+        """Set power supply current."""
+        devs = self._get_devices(psnames, psindices)
+        if isinstance(values, (int, float, bool)):
+            values = len(devs) * [values]
+        for i, dev in enumerate(devs):
+            dev.current = values[i]
+        return True
+
+    def check_current(
+            self, values, psnames=None, psindices=None,
+            atol=DEF_ATOL_CURRENT_RB):
+        """Check whether power supplies have desired current."""
+        if not self.connected:
+            return False
+        devs = self._get_devices(psnames, psindices)
+        impltd = _np.asarray([d.current for d in devs])
+        if isinstance(values, (int, float, bool)):
+            values = len(devs) * [values]
+        if _np.allclose(values, impltd, atol=atol):
+            return True
+        return False
+
+    def check_current_mon(
+            self, values, psnames=None, psindices=None,
+            atol=DEF_ATOL_CURRENT_MON):
+        """Check whether power supplies have desired implemented current."""
+        if not self.connected:
+            return False
+        devs = self._get_devices(psnames, psindices)
+        impltd = _np.asarray([d.current_mon for d in devs])
+        if isinstance(values, (int, float, bool)):
+            values = len(devs) * [values]
+        if _np.allclose(values, impltd, atol=atol):
+            return True
+        return False
+
     def set_invrespmat_row(self, values, psnames=None, psindices=None):
         """Command to set power supply correction coefficients value."""
         if not isinstance(values, (list, tuple, _np.ndarray)):
@@ -516,6 +724,8 @@ class FamFastCorrs(_Devices):
             self, values, psnames=None, psindices=None,
             atol=DEF_ATOL_INVRESPMATROW):
         """Check power supplies correction coefficients."""
+        if not self.connected:
+            return False
         if not isinstance(values, (list, tuple, _np.ndarray)):
             raise ValueError('Value must be iterable.')
         values = _np.asarray(values)
@@ -540,6 +750,8 @@ class FamFastCorrs(_Devices):
             self, values, psnames=None, psindices=None,
             atol=DEF_ATOL_FOFBACCGAIN):
         """Check whether power supplies have desired correction gain."""
+        if not self.connected:
+            return False
         if not isinstance(values, (list, tuple, _np.ndarray)):
             raise ValueError('Value must be iterable.')
         values = _np.asarray(values)
@@ -562,10 +774,61 @@ class FamFastCorrs(_Devices):
             self, values, psnames=None, psindices=None,
             timeout=DEF_TIMEOUT):
         """Check whether power supplies have desired freeze state."""
+        if not self.connected:
+            return False
         devs = self._get_devices(psnames, psindices)
+        if isinstance(values, (int, float, bool)):
+            values = len(devs) * [values]
+        values = list(values)
         return self._wait_devices_propty(
             devs, 'FOFBAccFreeze-Sts', values, timeout=timeout)
 
+    def set_fofbacc_satmax(self, values, psnames=None, psindices=None):
+        """Set power supply pre-accumulator max.saturation current."""
+        devs = self._get_devices(psnames, psindices)
+        if isinstance(values, (int, float, bool)):
+            values = len(devs) * [values]
+        for i, dev in enumerate(devs):
+            dev.fofbacc_satmax = values[i]
+        return True
+
+    def check_fofbacc_satmax(
+            self, values, psnames=None, psindices=None,
+            atol=DEF_ATOL_FOFBACCSAT):
+        """Check whether power supplies have desired max.saturation value."""
+        if not self.connected:
+            return False
+        devs = self._get_devices(psnames, psindices)
+        impltd = _np.asarray([d.fofbacc_satmax for d in devs])
+        if isinstance(values, (int, float, bool)):
+            values = len(devs) * [values]
+        if _np.allclose(values, impltd, atol=atol):
+            return True
+        return False
+
+    def set_fofbacc_satmin(self, values, psnames=None, psindices=None):
+        """Set power supply pre-accumulator min.saturation current."""
+        devs = self._get_devices(psnames, psindices)
+        if isinstance(values, (int, float, bool)):
+            values = len(devs) * [values]
+        for i, dev in enumerate(devs):
+            dev.fofbacc_satmin = values[i]
+        return True
+
+    def check_fofbacc_satmin(
+            self, values, psnames=None, psindices=None,
+            atol=DEF_ATOL_FOFBACCSAT):
+        """Check whether power supplies have desired min.saturation value."""
+        if not self.connected:
+            return False
+        devs = self._get_devices(psnames, psindices)
+        impltd = _np.asarray([d.fofbacc_satmin for d in devs])
+        if isinstance(values, (int, float, bool)):
+            values = len(devs) * [values]
+        if _np.allclose(values, impltd, atol=atol):
+            return True
+        return False
+
     def cmd_fofbacc_clear(self, psnames=None, psindices=None):
         """Send clear power supplies pre-accumulator."""
         for dev in self._get_devices(psnames, psindices):
@@ -596,13 +859,21 @@ class HLFOFB(_Device):
 
     _properties = (
         'LoopState-Sel', 'LoopState-Sts',
-        'LoopGain-SP', 'LoopGain-RB',
+        'LoopGainH-SP', 'LoopGainH-RB', 'LoopGainH-Mon',
+        'LoopGainV-SP', 'LoopGainV-RB', 'LoopGainV-Mon',
         'CorrStatus-Mon', 'CorrConfig-Cmd',
-        'CorrSetOpModeManual-Cmd', 'CorrSetAccFreezeDsbl-Cmd',
-        'CorrSetAccFreezeEnbl-Cmd', 'CorrSetAccClear-Cmd',
-        'FOFBCtrlStatus-Mon', 'FOFBCtrlSyncNet-Cmd',
+        'CorrSetPwrStateOn-Cmd', 'CorrSetOpModeManual-Cmd',
+        'CorrSetAccFreezeDsbl-Cmd', 'CorrSetAccFreezeEnbl-Cmd',
+        'CorrSetAccClear-Cmd', 'CorrSetCurrZero-Cmd',
+        'CHAccSatMax-SP', 'CHAccSatMax-RB',
+        'CVAccSatMax-SP', 'CVAccSatMax-RB',
+        'FOFBCtrlStatus-Mon', 'FOFBCtrlConfBPMId-Cmd',
+        'FOFBCtrlSyncNet-Cmd', 'FOFBCtrlSyncRefOrb-Cmd',
         'FOFBCtrlConfTFrameLen-Cmd', 'FOFBCtrlConfBPMLogTrg-Cmd',
+        'KickBufferSize-SP', 'KickBufferSize-RB', 'KickBufferSize-Mon',
+        'KickCH-Mon', 'KickCV-Mon',
         'RefOrbX-SP', 'RefOrbX-RB', 'RefOrbY-SP', 'RefOrbY-RB',
+        'RefOrbHwX-Mon', 'RefOrbHwY-Mon',
         'BPMXEnblList-SP', 'BPMXEnblList-RB',
         'BPMYEnblList-SP', 'BPMYEnblList-RB',
         'CHEnblList-SP', 'CHEnblList-RB',
@@ -652,13 +923,32 @@ class HLFOFB(_Device):
         self['LoopState-Sel'] = value
 
     @property
-    def loop_gain(self):
-        """Loop gain."""
-        return self['LoopGain-RB']
+    def loop_gain_h(self):
+        """Loop gain H."""
+        return self['LoopGainH-RB']
 
-    @loop_gain.setter
-    def loop_gain(self, value):
-        self['LoopGain-SP'] = value
+    @loop_gain_h.setter
+    def loop_gain_h(self, value):
+        self['LoopGainH-SP'] = value
+
+    @property
+    def loop_gain_h_mon(self):
+        """Implemented horizontal loop gain."""
+        return self['LoopGainH-Mon']
+
+    @property
+    def loop_gain_v(self):
+        """Loop gain V."""
+        return self['LoopGainV-RB']
+
+    @loop_gain_v.setter
+    def loop_gain_v(self, value):
+        self['LoopGainV-SP'] = value
+
+    @property
+    def loop_gain_v_mon(self):
+        """Implemented vertical loop gain."""
+        return self['LoopGainV-Mon']
 
     @property
     def corr_status(self):
@@ -670,6 +960,11 @@ class HLFOFB(_Device):
         self['CorrConfig-Cmd'] = 1
         return True
 
+    def cmd_corr_set_pwrstate_on(self):
+        """Command to set all corrector pwrstate to on."""
+        self['CorrSetCurrZero-Cmd'] = 1
+        return True
+
     def cmd_corr_set_opmode_manual(self):
         """Command to set all corrector opmode to manual."""
         self['CorrSetOpModeManual-Cmd'] = 1
@@ -690,16 +985,49 @@ class HLFOFB(_Device):
         self['CorrSetAccClear-Cmd'] = 1
         return True
 
+    def cmd_corr_set_current_zero(self):
+        """Command to set correctors current to zero."""
+        self['CorrSetCurrZero-Cmd'] = 1
+        return True
+
+    @property
+    def ch_accsatmax(self):
+        """CH accumulator maximum saturation limit."""
+        return self['CHAccSatMax-RB']
+
+    @ch_accsatmax.setter
+    def ch_accsatmax(self, value):
+        self['CHAccSatMax-SP'] = value
+
+    @property
+    def cv_accsatmax(self):
+        """CH accumulator maximum saturation limit."""
+        return self['CVAccSatMax-RB']
+
+    @cv_accsatmax.setter
+    def cv_accsatmax(self, value):
+        self['CVAccSatMax-SP'] = value
+
     @property
     def fofbctrl_status(self):
         """FOFB controller status."""
         return self['FOFBCtrlStatus-Mon']
 
+    def cmd_fofbctrl_conf_bpmid(self):
+        """Command to configure all FOFB DCC BPMIds."""
+        self['FOFBCtrlConfBPMId-Cmd'] = 1
+        return True
+
     def cmd_fofbctrl_syncnet(self):
         """Command to sync FOFB controller net."""
         self['FOFBCtrlSyncNet-Cmd'] = 1
         return True
 
+    def cmd_fofbctrl_syncreforb(self):
+        """Command to sync FOFB controller RefOrb."""
+        self['FOFBCtrlSyncRefOrb-Cmd'] = 1
+        return True
+
     def cmd_fofbctrl_conf_timeframelen(self):
         """Command to configure all FOFB controller TimeFrameLen."""
         self['FOFBCtrlConfTFrameLen-Cmd'] = 1
@@ -710,6 +1038,30 @@ class HLFOFB(_Device):
         self['FOFBCtrlConfBPMLogTrg-Cmd'] = 1
         return True
 
+    @property
+    def kick_buffer_size_mon(self):
+        """Return actual kicks buffer size."""
+        return self['KickBufferSize-Mon']
+
+    @property
+    def kick_buffer_size(self):
+        """Return kicks buffer size."""
+        return self['KickBufferSize-RB']
+
+    @kick_buffer_size.setter
+    def kick_buffer_size(self, value):
+        self['KickBufferSize-SP'] = max(1, int(value))
+
+    @property
+    def kickch(self):
+        """Return average of CH kicks."""
+        return self['KickCH-Mon']
+
+    @property
+    def kickcv(self):
+        """Return average of CV kicks."""
+        return self['KickCV-Mon']
+
     @property
     def refx(self):
         """RefOrb X."""
@@ -719,6 +1071,11 @@ class HLFOFB(_Device):
     def refx(self, value):
         self['RefOrbX-SP'] = value
 
+    @property
+    def refx_hw(self):
+        """RefOrb X in hardware units."""
+        return self['RefOrbHwX-Mon']
+
     @property
     def refy(self):
         """RefOrb Y."""
@@ -728,6 +1085,11 @@ class HLFOFB(_Device):
     def refy(self, value):
         self['RefOrbY-SP'] = value
 
+    @property
+    def refy_hw(self):
+        """RefOrb Y in hardware units."""
+        return self['RefOrbHwY-Mon']
+
     @property
     def bpmxenbl(self):
         """BPM X enable list."""
