@@ -85,10 +85,15 @@ class TLSOFB(_Device):
             raise NotImplementedError(devname)
 
         # SOFB object
-        self.data = data or SOFBFactory.create(devname[:2])
+        self._data = data or SOFBFactory.create(devname[:2])
         # call base class constructor
         super().__init__(devname, properties=self._properties)
 
+    @property
+    def data(self):
+        """."""
+        return self._data
+
     @property
     def opmode(self):
         """."""
@@ -97,12 +102,12 @@ class TLSOFB(_Device):
     @opmode.setter
     def opmode(self, value):
         self._enum_setter(
-            'SOFBMode-Sel', value, self.data.SOFBMode)
+            'SOFBMode-Sel', value, self._data.SOFBMode)
 
     @property
     def opmode_str(self):
         """."""
-        return self.data.SOFBMode._fields[self['SOFBMode-Sts']]
+        return self._data.SOFBMode._fields[self['SOFBMode-Sts']]
 
     @property
     def trigchannel(self):
@@ -112,12 +117,12 @@ class TLSOFB(_Device):
     @trigchannel.setter
     def trigchannel(self, value):
         self._enum_setter(
-            'TrigAcqChan-Sel', value, self.data.TrigAcqChan)
+            'TrigAcqChan-Sel', value, self._data.TrigAcqChan)
 
     @property
     def respmat(self):
         """."""
-        return self['RespMat-RB'].reshape(self.data.nr_bpms*2, -1)
+        return self['RespMat-RB'].reshape(self._data.nr_bpms*2, -1)
 
     @respmat.setter
     def respmat(self, mat):
@@ -127,12 +132,12 @@ class TLSOFB(_Device):
     @property
     def invrespmat(self):
         """."""
-        return self['InvRespMat-Mon'].reshape(-1, self.data.nr_bpms*2)
+        return self['InvRespMat-Mon'].reshape(-1, self._data.nr_bpms*2)
 
     @property
     def trigchannel_str(self):
         """."""
-        return self.data.TrigAcqChan._fields[self['TrigAcqChan-Sts']]
+        return self._data.TrigAcqChan._fields[self['TrigAcqChan-Sts']]
 
     @property
     def sp_trajx(self):
@@ -152,21 +157,21 @@ class TLSOFB(_Device):
     @property
     def trajx(self):
         """."""
-        if self.data.isring and self.opmode == self.data.SOFBMode.MultiTurn:
+        if self._data.isring and self.opmode == self._data.SOFBMode.MultiTurn:
             return self.mt_trajx_idx
         return self.sp_trajx
 
     @property
     def trajy(self):
         """."""
-        if self.data.isring and self.opmode == self.data.SOFBMode.MultiTurn:
+        if self._data.isring and self.opmode == self._data.SOFBMode.MultiTurn:
             return self.mt_trajy_idx
         return self.sp_trajy
 
     @property
     def sum(self):
         """."""
-        if self.data.isring and self.opmode == self.data.SOFBMode.MultiTurn:
+        if self._data.isring and self.opmode == self._data.SOFBMode.MultiTurn:
             return self.mt_sum_idx
         return self.sp_sum
 
@@ -367,22 +372,22 @@ class TLSOFB(_Device):
 
     def cmd_applycorr_ch(self):
         """."""
-        self['ApplyDelta-Cmd'] = self.data.ApplyDelta.CH
+        self['ApplyDelta-Cmd'] = self._data.ApplyDelta.CH
         return True
 
     def cmd_applycorr_cv(self):
         """."""
-        self['ApplyDelta-Cmd'] = self.data.ApplyDelta.CV
+        self['ApplyDelta-Cmd'] = self._data.ApplyDelta.CV
         return True
 
     def cmd_applycorr_rf(self):
         """."""
-        self['ApplyDelta-Cmd'] = self.data.ApplyDelta.RF
+        self['ApplyDelta-Cmd'] = self._data.ApplyDelta.RF
         return True
 
     def cmd_applycorr_all(self):
         """."""
-        self['ApplyDelta-Cmd'] = self.data.ApplyDelta.All
+        self['ApplyDelta-Cmd'] = self._data.ApplyDelta.All
         return True
 
     def cmd_measrespmat_start(self):
@@ -402,7 +407,7 @@ class TLSOFB(_Device):
 
     def cmd_change_opmode_to_multiturn(self, timeout=10):
         """."""
-        mode = self.data.SOFBMode.MultiTurn
+        mode = self._data.SOFBMode.MultiTurn
         self.opmode = mode
         ret = self._wait('SOFBMode-Sts', mode, timeout=timeout)
         if not ret:
@@ -412,7 +417,7 @@ class TLSOFB(_Device):
 
     def cmd_change_opmode_to_sloworb(self, timeout=10):
         """."""
-        mode = self.data.SOFBMode.SlowOrb
+        mode = self._data.SOFBMode.SlowOrb
         self.opmode = mode
         ret = self._wait('SOFBMode-Sts', mode, timeout=timeout)
         if not ret:
@@ -424,7 +429,7 @@ class TLSOFB(_Device):
         """."""
         self['TrigAcqCtrl-Sel'] = 'Start'
         ret = self._wait(
-            'TrigAcqCtrl-Sts', self.data.TrigAcqCtrl.Start, timeout=timeout)
+            'TrigAcqCtrl-Sts', self._data.TrigAcqCtrl.Start, timeout=timeout)
         if not ret:
             return False
         _time.sleep(0.6)  # Status PV updates at 2Hz
@@ -434,7 +439,7 @@ class TLSOFB(_Device):
         """."""
         self['TrigAcqCtrl-Sel'] = 'Stop'
         ret = self._wait(
-            'TrigAcqCtrl-Sts', self.data.TrigAcqCtrl.Stop, timeout=timeout)
+            'TrigAcqCtrl-Sts', self._data.TrigAcqCtrl.Stop, timeout=timeout)
         if not ret:
             return False
         _time.sleep(0.6)  # Status PV updates at 2Hz
@@ -444,7 +449,7 @@ class TLSOFB(_Device):
         """."""
         self['TrigAcqCtrl-Sel'] = 'Abort'
         ret = self._wait(
-            'TrigAcqCtrl-Sts', self.data.TrigAcqCtrl.Abort, timeout=timeout)
+            'TrigAcqCtrl-Sts', self._data.TrigAcqCtrl.Abort, timeout=timeout)
         if not ret:
             return False
         _time.sleep(0.6)  # Status PV updates at 2Hz
@@ -521,20 +526,20 @@ class TLSOFB(_Device):
     def cmd_turn_on_autocorr(self, timeout=None):
         """."""
         timeout = timeout or self._default_timeout
-        if self.autocorrsts == self.data.LoopState.Closed:
+        if self.autocorrsts == self._data.LoopState.Closed:
             return True
-        self['LoopState-Sel'] = self.data.LoopState.Closed
+        self['LoopState-Sel'] = self._data.LoopState.Closed
         return self._wait(
-            'LoopState-Sts', self.data.LoopState.Closed, timeout=timeout)
+            'LoopState-Sts', self._data.LoopState.Closed, timeout=timeout)
 
     def cmd_turn_off_autocorr(self, timeout=None):
         """."""
         timeout = timeout or self._default_timeout
-        if self.autocorrsts == self.data.LoopState.Open:
+        if self.autocorrsts == self._data.LoopState.Open:
             return True
-        self['LoopState-Sel'] = self.data.LoopState.Open
+        self['LoopState-Sel'] = self._data.LoopState.Open
         return self._wait(
-            'LoopState-Sts', self.data.LoopState.Open, timeout=timeout)
+            'LoopState-Sts', self._data.LoopState.Open, timeout=timeout)
 
     def wait_buffer(self, timeout=None):
         """."""
@@ -548,14 +553,14 @@ class TLSOFB(_Device):
         def_timeout = max(self._default_timeout_kick_apply, def_timeout)
         timeout = timeout or def_timeout
         return self._wait(
-            'ApplyDelta-Mon', self.data.ApplyDeltaMon.Applying,
+            'ApplyDelta-Mon', self._data.ApplyDeltaMon.Applying,
             timeout=timeout, comp='ne')
 
     def wait_respm_meas(self, timeout=None):
         """."""
         timeout = timeout or self._default_timeout_respm
         return self._wait(
-            'MeasRespMat-Mon', self.data.MeasRespMatMon.Measuring,
+            'MeasRespMat-Mon', self._data.MeasRespMatMon.Measuring,
             timeout=timeout, comp='ne')
 
     def wait_orb_status_ok(self, timeout=10):
@@ -576,37 +581,37 @@ class BOSOFB(TLSOFB):
     @property
     def mt_trajx(self):
         """."""
-        return self['MTurnOrbX-Mon'] if self.data.isring else None
+        return self['MTurnOrbX-Mon'] if self._data.isring else None
 
     @property
     def mt_trajy(self):
         """."""
-        return self['MTurnOrbY-Mon'] if self.data.isring else None
+        return self['MTurnOrbY-Mon'] if self._data.isring else None
 
     @property
     def mt_sum(self):
         """."""
-        return self['MTurnSum-Mon'] if self.data.isring else None
+        return self['MTurnSum-Mon'] if self._data.isring else None
 
     @property
     def mt_time(self):
         """."""
-        return self['MTurnTime-Mon'] if self.data.isring else None
+        return self['MTurnTime-Mon'] if self._data.isring else None
 
     @property
     def mt_trajx_idx(self):
         """."""
-        return self['MTurnIdxOrbX-Mon'] if self.data.isring else None
+        return self['MTurnIdxOrbX-Mon'] if self._data.isring else None
 
     @property
     def mt_trajy_idx(self):
         """."""
-        return self['MTurnIdxOrbY-Mon'] if self.data.isring else None
+        return self['MTurnIdxOrbY-Mon'] if self._data.isring else None
 
     @property
     def mt_sum_idx(self):
         """."""
-        return self['MTurnIdxSum-Mon'] if self.data.isring else None
+        return self['MTurnIdxSum-Mon'] if self._data.isring else None
 
     def cmd_mturn_acquire(self):
         """."""
@@ -649,12 +654,12 @@ class SISOFB(BOSOFB):
     @drivetype.setter
     def drivetype(self, value):
         self._enum_setter(
-            'DriveType-Sel', value, self.data.DriveType)
+            'DriveType-Sel', value, self._data.DriveType)
 
     @property
     def drivetype_str(self):
         """."""
-        return self.data.DriveType._fields[self['DriveType-Sts']]
+        return self._data.DriveType._fields[self['DriveType-Sts']]
 
     @property
     def drivefreqdivisor(self):
@@ -744,12 +749,12 @@ class SISOFB(BOSOFB):
     @property
     def orbx(self):
         """."""
-        return self['SlowOrbX-Mon'] if self.data.isring else None
+        return self['SlowOrbX-Mon'] if self._data.isring else None
 
     @property
     def orby(self):
         """."""
-        return self['SlowOrbY-Mon'] if self.data.isring else None
+        return self['SlowOrbY-Mon'] if self._data.isring else None
 
     @property
     def kickrf(self):
@@ -789,13 +794,13 @@ class SISOFB(BOSOFB):
     @property
     def rfenbl(self):
         """."""
-        dta = self.data
+        dta = self._data
         return self['RFEnbl-Sts'] if dta.acc_idx == dta.Rings.SI else None
 
     @rfenbl.setter
     def rfenbl(self, value):
         """."""
-        if self.data.acc_idx == self.data.Rings.SI:
+        if self._data.acc_idx == self._data.Rings.SI:
             self['RFEnbl-Sel'] = value
 
     @property
@@ -810,25 +815,25 @@ class SISOFB(BOSOFB):
     def cmd_turn_on_drive(self, timeout=None):
         """."""
         timeout = timeout or self._default_timeout
-        if self.drivests == self.data.DriveState.Closed:
+        if self.drivests == self._data.DriveState.Closed:
             return True
-        self['DriveState-Sel'] = self.data.DriveState.Closed
+        self['DriveState-Sel'] = self._data.DriveState.Closed
         return self._wait(
-            'DriveState-Sts', self.data.DriveState.Closed, timeout=timeout)
+            'DriveState-Sts', self._data.DriveState.Closed, timeout=timeout)
 
     def cmd_turn_off_drive(self, timeout=None):
         """."""
         timeout = timeout or self._default_timeout
-        if self.drivests == self.data.DriveState.Open:
+        if self.drivests == self._data.DriveState.Open:
             return True
-        self['DriveState-Sel'] = self.data.DriveState.Open
+        self['DriveState-Sel'] = self._data.DriveState.Open
         return self.wait_drive(timeout=timeout)
 
     def wait_drive(self, timeout=None):
         """."""
         timeout = timeout or self._default_timeout_respm
         return self._wait(
-            'DriveState-Sts', self.data.DriveState.Open, timeout=timeout)
+            'DriveState-Sts', self._data.DriveState.Open, timeout=timeout)
 
     @staticmethod
     def si_calculate_bumps(orbx, orby, subsec, agx=0, agy=0, psx=0, psy=0):
