@@ -263,6 +263,18 @@ class SOFBTLines(ConstTLines):
             'LoopPerfItersOk-Mon': {
                 'type': 'float', 'value': 0, 'unit': '%', 'prec': 3,
                 'lolim': -1, 'hilim': 100},
+            'LoopEffectiveRate-Mon': {
+                'type': 'float', 'value': 0, 'unit': 'Hz', 'prec': 3,
+                'lolim': 0, 'hilim': 100},
+            'LoopNumIters-Mon': {
+                'type': 'float', 'value': 0, 'unit': '#', 'prec': 0,
+                'lolim': 0, 'hilim': 100000},
+            'LoopPrintEveryNumIters-SP': {
+                'type': 'float', 'value': 1000, 'unit': '#', 'prec': 0,
+                'lolim': 1, 'hilim': 100000},
+            'LoopPrintEveryNumIters-RB': {
+                'type': 'float', 'value': 1000, 'unit': '#', 'prec': 0,
+                'lolim': 1, 'hilim': 100000},
             'LoopPerfItersTOut-Mon': {
                 'type': 'float', 'value': 0, 'unit': '%', 'prec': 3,
                 'lolim': -1, 'hilim': 100},
@@ -959,6 +971,48 @@ class SOFBSI(SOFBRings, ConstSI):
     def get_sofb_database(self, prefix=''):
         """Return SOFB database."""
         db_ring = {
+            'FOFBDownloadKicksPerc-SP': {
+                'type': 'float', 'value': 0.0, 'prec': 2, 'unit': '%',
+                'lolim': 0.0, 'hilim': 100},
+            'FOFBDownloadKicksPerc-RB': {
+                'type': 'float', 'value': 0.0, 'prec': 2, 'unit': '%',
+                'lolim': 0.0, 'hilim': 100},
+            'FOFBDownloadKicks-Sel': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
+            'FOFBDownloadKicks-Sts': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
+            'FOFBDownloadKicks-Mon': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
+            'FOFBUpdateRefOrb-Sel': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
+            'FOFBUpdateRefOrb-Sts': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
+            'FOFBUpdateRefOrb-Mon': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
+            'FOFBNullSpaceProj-Sel': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
+            'FOFBNullSpaceProj-Sts': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
+            'FOFBNullSpaceProj-Mon': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
+            'FOFBZeroDistortionAtBPMs-Sel': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
+            'FOFBZeroDistortionAtBPMs-Sts': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
+            'FOFBZeroDistortionAtBPMs-Mon': {
+                'type': 'enum', 'value': self.EnbldDsbld.Dsbld,
+                'enums': self.EnbldDsbld._fields},
             'DriveFreqDivisor-SP': {
                 'type': 'int', 'value': 12, 'unit': 'Div',
                 'lolim': 0, 'hilim': 1000},
