@@ -701,6 +701,10 @@ class EpicsOrbit(BaseOrbit):
             bpm.monit_sync_enbl = _csbpm.EnbldDsbld.disabled
             bpm.monit1_sync_enbl = _csbpm.EnbldDsbld.disabled
 
+        if self.acc == 'SI' and self.sofb.fofb.connected:
+            _time.sleep(0.2)
+            self.sofb.fofb.cmd_fofbctrl_syncnet()
+
     def set_trig_acq_control(self, value):
         """."""
         acqctrl = self._csorb.TrigAcqCtrl
